/*
 * (C) Copyright 2004
 * Robert Whaley, Applied Data Systems, Inc. rwhaley@applieddata.net
 *
 * (C) Copyright 2002
 * Kyle Harris, Nexus Technologies, Inc. kharris@nexus-tech.net
 *
 * (C) Copyright 2002
 * Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Marius Groeger <mgroeger@sysgo.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/arch/hardware.h>

/* ------------------------------------------------------------------------- */

typedef struct {
    u32 l : 5;
    u32 n : 4;
    u32 t : 1;
    u32 b : 1;
    u32 a : 1;
    u32 k1db2 : 1;
    u32 k0db2 : 1;
    u32 k0db4 : 1;
    u32 ht : 1;
    u32 res : 16; //Reserved for future usage
} speed_t;

#ifdef CFG_DEEP_DEBUG
int dump_registers(void)
{
    printf("CCCR = 0x%08x\n", CCCR);
    printf("MSC0 = 0x%08x\n", MSC0);
    printf("MSC1 = 0x%08x\n", MSC1);
    printf("MSC2 = 0x%08x\n", MSC2);

    printf("GPDR0 = 0x%08x\n", GPDR0);
    printf("GPDR1 = 0x%08x\n", GPDR1);
    printf("GPDR2 = 0x%08x\n", GPDR2);
    printf("GPDR3 = 0x%08x\n", GPDR3);

    printf("GPLR0 = 0x%08x\n", GPLR0);
    printf("GPLR1 = 0x%08x\n", GPLR1);
    printf("GPLR2 = 0x%08x\n", GPLR2);
    printf("GPLR3 = 0x%08x\n", GPLR3);

    printf("GAFR0_L = 0x%08x\n", GAFR0_L);
    printf("GAFR0_U = 0x%08x\n", GAFR0_U);
    printf("GAFR1_L = 0x%08x\n", GAFR1_L);
    printf("GAFR1_U = 0x%08x\n", GAFR1_U);
    printf("GAFR2_L = 0x%08x\n", GAFR2_L);
    printf("GAFR2_U = 0x%08x\n", GAFR2_U);
    printf("GAFR3_L = 0x%08x\n", GAFR3_L);
    printf("GAFR3_U = 0x%08x\n", GAFR3_U);
}
#endif

/*
 * Miscelaneous platform dependent initialisations
 */

int board_init (void)
{
	DECLARE_GLOBAL_DATA_PTR;

	/* memory and cpu-speed are setup before relocation */
	/* so we do _nothing_ here */

	/* arch number of vpac270 */
	gd->bd->bi_arch_number = MACH_TYPE_VPAC270;

	/* adress of boot parameters */
	gd->bd->bi_boot_params = 0xa0000100;

	return 0;
}

#ifdef BOARD_LATE_INIT
int board_late_init(void)
{
	DECLARE_GLOBAL_DATA_PTR;
    unsigned char mac_addr[6] = {0, 0, 0, 0, 0, 0};
    unsigned char mac_addrff[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
    char env_str[64];
    u32 cfg_addr = CFG_ENV_ADDR + CFG_ENV_SECT_SIZE - CFG_FIX_PARAM_SIZE;
    union {
        u32 speed_val;
        speed_t speed_param;
    } speed_u;
    int env_changed = 0;
    char *speed_str;

//	setenv("stdout", "serial");
//	setenv("stderr", "serial");

    /* handle production parameter config block */
    if (memcmp(mac_addrff, (unsigned char *)cfg_addr, 6) == 0
           || memcmp(mac_addr, (unsigned char *)cfg_addr, 6) == 0) {
//        printf("Missing Colibri config block\n");
        return 0;
    }

    /* set Ethernet address from config block if not already set */
    if (memcmp(mac_addr, gd->bd->bi_enetaddr, 6) == 0) {
        memcpy(mac_addr, (unsigned char *)cfg_addr, 6);
        sprintf(env_str, "%02x:%02x:%02x:%02x:%02x:%02x",
                mac_addr[0], mac_addr[1], mac_addr[2],
                mac_addr[3], mac_addr[4], mac_addr[5]);
        setenv("ethaddr", env_str);
        env_changed = 1;
    }

    /* set CPU speed from config block if not already set */
    if ((speed_str = getenv("pxaspeed")) == NULL) {
        speed_u.speed_param = *((speed_t *)(cfg_addr+8));
        if (speed_u.speed_param.l < 7 || speed_u.speed_param.l > 16) {
            printf("illegal CPU speed parameter; using safe default\n");
            speed_u.speed_val = 0x0670;
        }
        sprintf(env_str, "0x%08x", speed_u.speed_param);
        setenv("pxaspeed", env_str);
        env_changed = 1;
    }
    else
    {
        speed_u.speed_val = simple_strtol(speed_str, NULL, 0);
    }
    pxa27x_set_core_freq(speed_u.speed_param.l, speed_u.speed_param.n,
                         speed_u.speed_param.t, speed_u.speed_param.b,
                         1); // for specific steppings only!
                         //speed_u.speed_param.a);

    if (env_changed) saveenv();

    printf("CPU speed: %dkHz\n", pxa27x_get_freq());

#ifdef CFG_DEEP_DEBUG
    dump_registers();
#endif

	return 0;
}
#endif

int dram_init (void)
{
	DECLARE_GLOBAL_DATA_PTR;

	u32 size, mdcnfg = MDCNFG;
	
	gd->bd->bi_dram[0].start = PHYS_SDRAM_1;
	gd->bd->bi_dram[1].start = PHYS_SDRAM_2;
//	gd->bd->bi_dram[2].start = PHYS_SDRAM_3;
//	gd->bd->bi_dram[3].start = PHYS_SDRAM_4;

	/* bank 0 partition 0/1 size */
	size = 1 << ( ((mdcnfg >> 3) & 3) + ((mdcnfg >> 5) & 3) + 23);

	if( mdcnfg & 0x04)
		size >>= 1;
	
	gd->bd->bi_dram[0].size  = (mdcnfg & 0x03)? size : 0;

	/* bank 1 partition 2/3 size */
	size = 1 << ( ((mdcnfg >> 19) & 3) + ((mdcnfg >> 21) & 3) + 23);

	if( mdcnfg & 0x40000)
		size >>= 1;
	
	gd->bd->bi_dram[1].size  = (mdcnfg & 0x30000)? size : 0;

	return 0;
}
